/* Ext.ux.HistoryProvider
	- provide saving state to the browser history
	- fire url change events: back/forward/refresh buttons clicked
	- encapsulate the work with Ext.History
 * Author: Petrova Olga. gyoushe@gmail.com
*/
Ext.ux.HistoryProvider = function(config) {
    Ext.ux.HistoryProvider.superclass.constructor.call(this);
    this.fieldId = 'x-history-field';
    this.iframeId = 'x-history-frame';
    this.addEvents("statechange");
    this.state = {};
    Ext.apply(this, config);
};

Ext.extend(Ext.ux.HistoryProvider, Ext.state.Provider, {
    decodeValue: function(value) {
        return value ? (Ext.util.JSON.decode(decodeURIComponent(value))) : {};
    },

    encodeValue: function(value) {
        var result = value ? (Ext.util.JSON.encode(value)) : '';
        if (result == '{}') result = '';
        return result;
    },

    init: function() {
    	if (this.listeners)
            if (this.listeners['statechange']) this.addListener('statechange', this.listeners['statechange']);
        Ext.History.fieldId = this.fieldId;
        Ext.History.iframeId = this.iframeId;
        var initToken = this.readUrl();
        Ext.History.init(function(){
        	if(Ext.isIE)Ext.History.add(initToken, false);
        	else Ext.History.add(initToken);
        });
        if (initToken != '')this.changeHistory(initToken);
        Ext.History.on('change', function(scope) {
            return function(token) { scope.changeHistory.call(scope, token); }
        } (this));
    },

    set: function(name, value) {
        if (typeof name == "undefined" || name === null) return;
        if (typeof value == "undefined" || value === null) return;
        this.setHistory(name, value);
        Ext.ux.HistoryProvider.superclass.set.call(this, name, value);
    },

    get: function(name) {
        return this.getHistory(name);
    },

    clear: function(name) {
        Ext.ux.HistoryProvider.superclass.clear.call(this, name);
    },

    readUrl: function() {
        var url = document.location.hash.replace(/#/g, '');
        return url;
    },

	//save to the history the state [value] of the element [name]
    setHistory: function(name, value) {
        if (this.state[name] != value) {
            this.state[name] = value;
            Ext.History.add(this.encodeValue(this.state));
        }
    },

    getHistory: function(name) {
        return this.state[name];
    },

	//fire the url change event
    changeHistory: function(value) {
        var newState = this.decodeValue(value);
        var previousState = this.state;
        if (this.encodeValue(previousState) != this.encodeValue(newState)) {
        	this.state = newState;
        	if(this.encodeValue(newState) == "")
            	this.fireEvent('statechange', this, "", newState);
            else
	        	for (element in newState)
	            	if (this.encodeValue(newState[element]) != this.encodeValue(previousState[element]))
	                	this.fireEvent('statechange', this, element, newState[element]);
        }
    }
});