/* Ext.ux.HistoryTree - tree with history
 * Author: Petrova Olga. gyoushe@gmail.com
*/
Ext.ux.HistoryTree = Ext.extend(Ext.tree.TreePanel, {
    //state synchronizing flag
    isStateSynchronizing: 0,
    currentState: {},
    initComponent: function() {
        Ext.ux.HistoryTree.superclass.initComponent.call(this);
        var provider = this.hprovider;
        if(!provider)return;
        
        if(this.historyId == null)this.historyId = this.id;
        
        var self = this;
        var treeChangeHandler = function() {
        	if (self.isStateSynchronizing > 0) {
        		self.isStateSynchronizing--;
	            return;
            }
            provider.set(self.historyId, self.getState());
        }

		//add tree change handler to the expand/collapse/select event
        this.on('expandnode', function(node) {
        	if((node == self.root) && (!self.rootVisible))return;
        	treeChangeHandler();
        });
        this.on('collapsenode', function(node) {
        	treeChangeHandler();
        });
        (this.getSelectionModel()).on('selectionchange', function(model, node) {
        	treeChangeHandler();
        });

        this.root.on('expand',
            function() {
                provider.on('statechange', function(el, name, state) {
                	if((name != "")&&(name != self.historyId)) return;
                    self.applyState(state);
                });
                provider.init();
            }
        );
        if(this.rootVisible)this.root.expand();
    },

	//get the current state of the tree in the string
	//example:  {"1":{"2":{},"3":{"6":{},sel:"7"}},"5":{}}
	//"1","5" - number of expanded nodes level 1; "2","3" - number of expanded children nodes of the node "1",
	//"6" -  number of expanded children node of the node "3", "7" - number of the selected children node of node "3"
    getState: function() {
    	if(this.isStateSynchronizing > 0) return this.currentState;
        var state = new Object();
        var currentNode = this.root;
        var getStateHandler = function(state, node) {
            for (index in node.childNodes) {
                var childNode = node.childNodes[index];
                if (!childNode.id) continue;
                //selected node
                if (childNode.isSelected())state['sel'] = index;
                //expanded nodes
                if (childNode.isExpanded() && childNode.hasChildNodes()) {
                    state[index] = new Object();
                    getStateHandler(state[index], childNode);
                }
            }
        };
        getStateHandler(state, currentNode);
        this.currentState = state;
        return this.currentState;
    },
    
    //apply the state to the node
	applyStateToNode: function(state, node) {
		var self = this;
		for (index in node.childNodes) {
	        var childNode = node.childNodes[index];
	        if (childNode.id == null) continue;
	        var childState = state[index];
	        
	        //select
	        if ((state['sel'] !== undefined) && (state['sel'] == index)) 
	        {
	        	self.isStateSynchronizing++;
	        	childNode.select();
	        }
	        else if (childNode.isSelected()) 
	        {
	        	self.isStateSynchronizing++;
	        	childNode.unselect();
	        }
	
			//expand
	        var expanded = (childState != null) ? true : false;
	        if (expanded) {
	            var lnode = childNode, lstate = childState;
	            var fnApplyStateToNode = function(fstate, fnode) {
	            		 return function() {
		                 	fnode.removeListener("expand", fnApplyStateToNode);
		                 	self.applyStateToNode(fstate, fnode);
		                 }
		        } (lstate, lnode);
	            if (!childNode.isExpanded()) {
	            	self.isStateSynchronizing++;
	            	childNode.on("expand", fnApplyStateToNode, childNode);
	                childNode.expand();
	            }
	            else fnApplyStateToNode();
	        }
	        //collapse
	        else if (childNode.isExpanded()) {
	            	self.isStateSynchronizing++;
	               	childNode.collapse();
	        }
	    }
	
	},

	//apply state to the tree
    applyState: function(state) {
    	if (state == null) state = {};
        if (state == this.currentState) return;
        this.currentState = state;
        
        this.isStateSynchronizing = 1;
        this.applyStateToNode(this.currentState, this.root);
        this.isStateSynchronizing--;
    }


});